@@ -1387,7 +1387,7 @@ void ShowReplyToChatBox(
 		history->setLocalDraft(std::make_unique<Data::Draft>(
 			textWithTags,
 			reply,
-			SuggestPostOptions(),
+			SuggestOptions(),
 			cursor,
 			Data::WebPageDraft()));
 		history->clearLocalEditDraft(topicRootId, monoforumPeerId);
