@@ -13,6 +13,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "api/api_sensitive_content.h"
 #include "api/api_global_privacy.h"
 #include "api/api_websites.h"
+#include "data/components/passkeys.h"
 #include "settings/cloud_password/settings_cloud_password_email_confirm.h"
 #include "settings/cloud_password/settings_cloud_password_input.h"
 #include "settings/cloud_password/settings_cloud_password_login_email.h"
@@ -21,6 +22,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "settings/settings_blocked_peers.h"
 #include "settings/settings_global_ttl.h"
 #include "settings/settings_local_passcode.h"
+#include "settings/settings_passkeys.h"
 #include "settings/settings_premium.h" // Settings::ShowPremium.
 #include "settings/settings_privacy_controllers.h"
 #include "settings/settings_websites.h"
@@ -32,6 +34,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/application.h"
 #include "core/core_settings.h"
 #include "history/view/media/history_view_media_common.h"
+#include "ui/basic_click_handlers.h"
 #include "ui/chat/chat_style.h"
 #include "ui/effects/premium_graphics.h"
 #include "ui/effects/premium_top_bar.h"
@@ -457,6 +460,10 @@ void SetupPrivacy(
 		tr::lng_settings_bio_privacy(),
 		Key::About,
 		[] { return std::make_unique<AboutPrivacyController>(); });
+	add(
+		tr::lng_settings_saved_music_privacy(),
+		Key::SavedMusic,
+		[] { return std::make_unique<SavedMusicPrivacyController>(); });
 	add(
 		tr::lng_settings_groups_invite(),
 		Key::Invites,
@@ -540,7 +547,7 @@ void SetupCloudPassword(
 		tr::lng_settings_cloud_password_start_title(),
 		std::move(label),
 		st::settingsButton,
-		{ &st::menuIconPermissions }
+		{ &st::menuIcon2SV }
 	)->addClickHandler([=, passwordState = base::duplicate(passwordState)] {
 		const auto state = rpl::variable<PasswordState>(
 			base::duplicate(passwordState)).current();
@@ -569,6 +576,52 @@ void SetupCloudPassword(
 	session->api().cloudPassword().reload();
 }
 
+void SetupPasskeys(
+		not_null<Window::SessionController*> controller,
+		not_null<Ui::VerticalLayout*> container) {
+	const auto session = &controller->session();
+	if (!session->passkeys().possible()) {
+		return;
+	}
+	auto label = rpl::combine(
+		tr::lng_profile_loading(),
+		(rpl::single(rpl::empty_value())
+			| rpl::then(session->passkeys().requestList())) | rpl::map([=] {
+			return session->passkeys().list().size();
+		})
+	) | rpl::map([=](const QString &loading, int count) {
+		return !session->passkeys().listKnown()
+			? loading
+			: count
+			? QString::number(count)
+			: tr::lng_settings_cloud_password_off(tr::now);
+	});
+	AddButtonWithLabel(
+		container,
+		tr::lng_settings_passkeys_title(),
+		std::move(label),
+		st::settingsButton,
+		{ &st::menuIconPermissions }
+	)->addClickHandler([=] {
+		if (!session->passkeys().listKnown()) {
+			return;
+		}
+		const auto count = session->passkeys().list().size();
+		if (count == 0) {
+			controller->show(Box([=](not_null<Ui::GenericBox*> box) {
+				PasskeysNoneBox(box, session);
+				box->boxClosing() | rpl::start_with_next([=] {
+					if (session->passkeys().list().size()) {
+						controller->showSettings(PasskeysId());
+					}
+				}, box->lifetime());
+			}));
+		} else {
+			controller->showSettings(PasskeysId());
+		}
+	});
+}
+
 void SetupLoginEmail(
 		not_null<Window::SessionController*> controller,
 		not_null<Ui::VerticalLayout*> container,
@@ -604,30 +657,8 @@ void SetupLoginEmail(
 		{ &st::menuIconRecoveryEmail });
 	CreateRightLabel(button, std::move(label), st, std::move(text));
 
-	button->addClickHandler([=, email = std::move(email)] {
-		controller->uiShow()->show(Box([=](not_null<Ui::GenericBox*> box) {
-			{
-				box->getDelegate()->setTitle(rpl::duplicate(
-					email
-				) | rpl::map(Ui::Text::WrapEmailPattern));
-				for (const auto &child : ranges::views::reverse(
-						box->parentWidget()->children())) {
-					if (child && child->isWidgetType()) {
-						(static_cast<QWidget*>(child))->setAttribute(
-							Qt::WA_TransparentForMouseEvents);
-						break;
-					}
-				}
-			}
-			Ui::ConfirmBox(box, Ui::ConfirmBoxArgs{
-				.text = tr::lng_settings_cloud_login_email_box_about(),
-				.confirmed = [=](Fn<void()> close) {
-					showOther(CloudLoginEmailId());
-					close();
-				},
-				.confirmText = tr::lng_settings_cloud_login_email_box_ok(),
-			});
-		}));
+	button->addClickHandler([=] {
+		UrlClickHandler::Open(u"tg://settings/login_email"_q);
 	});
 
 	const auto reloadOnActivation = [=](Qt::ApplicationState state) {
@@ -947,6 +978,7 @@ void SetupSecurity(
 		rpl::duplicate(updateTrigger),
 		showOther);
 	SetupLocalPasscode(controller, container, showOther);
+	SetupPasskeys(controller, container);
 	SetupLoginEmail(controller, container, showOther);
 	SetupBlockedList(
 		controller,
